// main.h

const char version[64] = "\nJed Margolin's LCD/Clock Tests   7/4/2024";  // for uart

// GPIO Pins
#define LCD_E  10
#define LCD_RS 11
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 8
#define LCD_D7 9

#define PB1 14
#define PB2 13
#define PB3 12
#define SW  15

#define CHANNEL 3    // RF Channel
#define LED1 2
#define LED2 27

#define TXEN 21    // nrf905 Transmit Enable
#define TRX_CE 20  // enable nrf905 chip for transmit/receive
#define AM 28      // Address Match
#define DR 22      // Data Ready
#define CSN905 17  // nrf906 CSN - pin 22
#define BUF_LEN  33

#define UART_TX_PIN 0
#define UART_RX_PIN 1
#define UART_ID uart0
#define BAUD_RATE 115200

// Pushbutton switches
extern struct myswitch switch1;   // also defined and used in clock.h
extern struct myswitch switch2;
extern struct myswitch switch3;

extern void do_switch(unsigned char, struct myswitch *); 
//extern unsigned char timer_flag;

// for nrf905
// The first byte is the instruction and the address start for nrf905, and example is WC1[0]
unsigned char WC1[11] = { 0x00, 0x70, 0x0C, 0x44, 0x20, 0x20, 0x61, 0x62, 0x63, 0x64, 0xDB };  // Test 433.6 MHz
unsigned char TXADR1[5] = { 0x22, 0x61, 0x62, 0x63, 0x64 };  // TXADR[0] is the instruction

unsigned char WC2[11] = { 0x00, 0x6E, 0x0C, 0x44, 0x20, 0x20, 0x54, 0x53, 0x52, 0x51, 0xDB };  // Garage 433.4 MHz
unsigned char TXADR2[5] = { 0x22, 0x54, 0x53, 0x52, 0x51 };  // TXADR[0] is the instruction

unsigned char spi_out_buffer[BUF_LEN];   // out_buffer[0] is the instruction
unsigned char spi_in_buffer[BUF_LEN];    // in_buffer[0] is the status byte

// Messages - structure is duplicated in clock.h
struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};
//----------------------------------------------
const struct messages lcd_msg1 = {0x81, 14,"Jed Margolin's"};
const struct messages lcd_msg2 = {0xC0, 15, "LCD/Clock Tests"};
const struct messages lcd_msg3 = {0x83, 8, "7/4/2024"};
const struct messages lcd_msg4 = {0x83, 6, "Line 1"};
const struct messages lcd_msg5 = {0xC3, 6, "Line 2"};
const struct messages lcd_msg6 = {0x84, 6, "nrf905"};
const struct messages lcd_msg7 = {0x83, 5, "Ready"};
const struct messages lcd_msg8 = {0x80, 4, "Hex8"};
const struct messages lcd_msg9 = {0xC0, 5, "Hex16"};
const struct messages lcd_msg10 = {0x80, 5, "Hex32"};
const struct messages lcd_msg11 = {0x80, 7, "uint8_t"};
const struct messages lcd_msg12 = {0xC0, 6, "int8_t"};
const struct messages lcd_msg13 = {0x80, 8, "uint16_t"};
const struct messages lcd_msg14 = {0xC0, 7, "int16_t"};
const struct messages lcd_msg15 = {0x80, 8, "uint32_t"};
const struct messages lcd_msg16 = {0x80, 7, "int32_t"};
const struct messages lcd_msg17 = {0x83, 9, "Press PB1"};
//=================================================================
extern void SetChrMode(void);
extern void SetCmdMode(void);
extern void lcd_init(void);
extern void domsg(const struct messages *);
extern void lcd_wr_data(unsigned char);
extern void lcd_wr_instr(unsigned char);
extern void lcd_text(char *);
extern void lcd_byte(char);
extern void lcd_wr_time(unsigned int, unsigned int);

//   i2c_write_blocking(i2c_default, DSADR, out_buffer, nbytes_out, false);
uint8_t out_buffer[10];   // first byte is register address

extern unsigned char nflag;
struct repeating_timer timer;

void do_lcd(void);
extern void do_clock(void);
//=======================================================
unsigned char mode;   // 0 = RX; 1 = clock
unsigned char data_received, irq_flag, period, secs_flag;
unsigned int save_count, irq_count, ts_number;
unsigned char nflag;

char nstr[128];
char line[256];
//=================================================================
void my_itoa(unsigned int, char *);
void my_itoa2(unsigned int, char *);

void line_fill_bin(unsigned char,char *, unsigned char, unsigned char);
void line_fill(unsigned char, char *, unsigned char, unsigned char);

void test_uart(void);
void log1_time_uart(void);
void log2_time_uart(void);
void my_hextoa2(unsigned char, char *);
void my_hextoa8(uint32_t, char *); 
void LED_break(void);
void nrf905_init(void);
extern void clear_lcd(unsigned char);

// From clock.c
#define SECS   0
#define MINS   1
#define HOURS  2
#define DAY    3
#define DATE   4
#define MONTH  5
#define YEAR   6

extern const char day_array[7][3];
extern const char *month_name[12];
extern void i2c_get_time(void);
extern unsigned char time_buffer[];
extern const struct messages msg1, msg2, msg3,msg15;
//===============================================================
unsigned char uart_char;

void keyboard_echo(void);
void do_uart_RX(void);

uint32_t ints;
//================================================
void uint8_bcd_str(unsigned char, char *);   // unsigned char to BCD string    
void uint16_bcd_str(unsigned int, char *);   // unsigned int to BCD string
void uint32_bcd_str(uint32_t, char *);       // unsigned long to BCD string

void sint8_bcd_str(int8_t, char *);          // signed char to BCD string    
void sint16_bcd_str(int, char *);            // signed int to BCD string
void sint32_bcd_str(int32_t, char *);       // signed long to BCD string

void uint8_hex_str(unsigned char, char *);   // unsigned char to hex string    
void uint16_hex_str(unsigned int, char *);   // unsigned int to hex string
void uint32_hex_str(unsigned long, char *);  // unsigned long to hex string

void uint8_bcd_lcd(uint8_t, uint8_t);            // unsigned char to BCD to lcd    
void uint16_bcd_lcd(uint16_t, uint8_t);          // unsigned int to BCD to lcd
void uint32_bcd_lcd(uint32_t bin, uint8_t pos);  // unsigned long to BCD lcd

void sint8_bcd_lcd(int8_t bin, uint8_t pos);     // signed char to BCD to lcd    
void sint16_bcd_lcd(int16_t, uint8_t);           // signed int to BCD to lcd
void sint32_bcd_lcd(int32_t bin, uint8_t pos);   // signed long to BCD to lcd

void uint8_hex_lcd(uint8_t, uint8_t);    // unsigned char to hex to lcd    
void uint16_hex_lcd(uint16_t, uint8_t);  // unsigned int to hex to lcd
void uint32_hex_lcd(uint32_t, uint8_t);  // unsigned long to hex to lcd

void wait_pb1(void);

unsigned char val1;
uint16_t val2;
uint32_t val3;
unsigned char val4;
char val5;
uint16_t val6;
int val7;
uint32_t val8;
int32_t val9;

// For reading, debouncing, and processing the switches
struct myswitch
{
    unsigned char debounce;     // debounce byte
    unsigned char state;        // debounced state of the switch
    unsigned char old;          // old state
    unsigned char press;        // switch has been pressed
    unsigned char release;      // switch has been released
};
// Declare and initialize the switches
extern struct myswitch switch1;
extern struct myswitch switch2;
extern struct myswitch switch3;

/*
Name 	            C++ 	   Bytes 	Range

bool 		                    1 	true or false
signed char 	    int8_t 	    1 	-128 to 127
unsigned char 	    uint8_t 	1 	0 to 255
short int 	        int16_t 	2 	-32768 to 32767
unsigned short int 	uint16_t 	2 	0 to 65535
int 	            int32_t 	4 	-2147483648 to 2147483647
unsigned int 	    uint32_t 	4 	0 to 4294967295
long int 	        int32_t 	4 	-2147483648 to 2147483647
unsigned long int 	uint32_t 	4 	0 to 4294967295
long long 	        int64_t 	8 	−9,223,372,036,854,775,808 to 9,223,372,036,854,775,807
unsigned long long 	uint64_t 	8 	0 to 18,446,744,073,709,551,615
float 		                    4 	+/- 3.4e +/- 38 (~7 digits)
double 		                    8 	+/- 1.7e +/- 308 (~15 digits)
long double 		            8 	+/- 1.7e +/- 308 (~15 digits)
wchar_t 		            2 or 4  1 wide character
*/
